"""A setuptools based setup module.
See:
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
"""

# Always prefer setuptools over distutils
from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
from os import path

setup(
    name='artifactory-lib',

    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    # Step 7: use from SCM https://pypi.python.org/pypi/setuptools_scm
    use_scm_version=True,
    setup_requires=['setuptools_scm'],

    description='Python library for interacting with Artifactory API',
    url='https://github.com/gugahoi/artifactory_python',
    author='Gustavo Hoirisch',
    author_email='gugahoi@gmail.com',
    license='MIT',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',

        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',

        'License :: OSI Approved :: MIT License',

        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],

    # What does your project relate to?
    keywords='artifactory management library',
    py_modules=["Artifactory"],
    install_requires=['requests']
)



import requests

class Artifactory(object):
    """Artifactory Management Tool"""
    def __init__(self, username, apikey, base_url='https://localhost:8080/artifactory'):
        super(Artifactory, self).__init__()
        self.username = username
        self.apikey   = apikey
        self.base_url = base_url

    def __str__(self):
        return 'Artifactory: %s @ %s' % (self.username, self.base_url)

    def __repr__(self):
        return '<Artifactory user:%s url:%s>' % (self.username, self.base_url)

    ''' Internal Setters '''
    def set_username(self, username):
        self.username = username

    def set_apikey(self, apikey):
        self.apikey = apikey

    def set_base_url(self, base_url):
        self.base_url = base_url

    ''' Helper functions '''
    def build_url(self, endpoint):
        return ''.join([
            self.base_url,
            '/api',
            endpoint
        ])

    def get(self, endpoint):
        r = requests.get(self.build_url(endpoint), auth=(self.username, self.apikey))
        if r.status_code is 200:
            return r.json()

    def post(self, endpoint):
        r = requests.post(self.build_url(endpoint), auth=(self.username, self.apikey))
        if r.status_code is 200:
            return r.json()

    def put(self, endpoint, data={}):
        r = requests.put(
            self.build_url(endpoint),
            auth=(self.username, self.apikey),
            json=data
        )
        if r.status_code is 200:
            return r.json()

    def delete(self, endpoint):
        r = requests.delete(self.build_url(endpoint), auth=(self.username, self.apikey))
        if r.status_code is 200:
            return r.text

    '''
    Security Section
    https://www.jfrog.com/confluence/display/RTF/Artifactory+REST+API#ArtifactoryRESTAPI-SECURITY ###
    '''
    def get_users(self):
        return self.get('/security/users')

    def get_user(self, username):
        return self.get('/security/users/%s' % username)

    def create_user(self, username):
        return self.put('/security/users/%s' % username.lower(), {
            'email': '%s@email.com' % username,
            'password': 'password'
        })

    def delete_user(self, username):
        return self.delete('/security/users/%s' % username.lower())

    def get_api_key(self):
        return self.get('/security/apiKey')

    def create_api_key(self):
        return self.post('/security/apiKey')

    def regenerate_api_key(self):
        return self.put('/security/apiKey')

    def revoke_api_key(self):
        return self.delete('/security/apiKey')
